/******************************************************************************
* simButtonsLogic.js                                                          *
*                                                                             *
* Copyright 2003-2004 ISY Intellect.                                          *
*                                                                             *
* Functions for Sim Buttons Logic.                                            *
******************************************************************************/

//----------------------------------------------------------------------------
// Variables for SimButton objects.
//----------------------------------------------------------------------------

// Constants. Button's ids in simButtons array.
var homeButton      = 0;
var selectButton    = 1;
var playButton      = 2;
var stopButton      = 3;
var nextStepButton  = 4;
var rewindButton    = 5;
var drillDownButton = 6;
var setDelayButton  = 7;
var skipButton      = 8;

var simButtons = new Array();

var drillName = "drill";

simButtons[homeButton]      = new SimButton("menu", "onHomeClick();", "images/Home.gif", "images/Home.gif", "images/HomeOver.gif", "images/HomeClick.gif");
simButtons[selectButton]    = new SimButton("selSim", "onSelSimClick();", "images/Select.gif", "images/Select.gif", "images/SelectOver.gif", "images/SelectClick.gif");
simButtons[playButton]      = new SimButton("play", "onPlayClick();", "images/playButton.gif", "images/playButtonDis.gif", "images/playButtonOver.gif", "images/playButtonClicked.gif");
simButtons[stopButton]      = new SimButton("stop", "onStopClick();", "images/stopButton.gif", "images/stopButtonDis.gif", "images/stopButtonOver.gif", "images/stopButtonClicked.gif");
simButtons[nextStepButton]  = new SimButton("step", "onStepClick();", "images/stepButton.gif", "images/stepButtonDis.gif", "images/stepButtonOver.gif", "images/stepButtonClicked.gif");
simButtons[rewindButton]    = new SimButton("rewind", "onRewindClick();", "images/rewButton.gif", "images/rewButtonDis.gif", "images/rewButtonOver.gif", "images/rewButtonClicked.gif");
simButtons[drillDownButton] = new SimButton(drillName, "onDrillClick();", "images/Drill.gif", "images/DrillDis.gif", "images/DrillOver.gif", "images/DrillClick.gif", "images/DrillClickOff.gif");
simButtons[setDelayButton]  = new SimButton("delay", "", "images/delayButton.gif", "images/delayButtonDis.gif", "images/delayButton.gif", "images/delayButton.gif");
simButtons[skipButton]      = new SimButton("skip", "onSkipClick();", "images/skipButton.gif", "images/skipButtonDis.gif", "images/skipButtonOver.gif", "images/skipButtonClicked.gif");

var menuFrameName = "menuFrame";
var mainFrameName = "mainFrame";

//----------------------------------------------------------------------------
// Constructor for SimButton object.
//----------------------------------------------------------------------------

function SimButton(buttonName, script, imgEnabled, imgDisabled, imgOnMouseOver, imgSelected, imgSelectedOff)
{
  // member attributes

  this.buttonName = buttonName;
  this.script= script;
  this.imgEnabled = imgEnabled;
  this.imgDisabled = imgDisabled;
  this.imgOnMouseOver = imgOnMouseOver;
  this.imgSelected = imgSelected;
  this.imgSelectedOff = imgSelectedOff;

  this.active = false;
  this.selected = false;

  // member functions

  this.setSelected = setSelected;
  this.setActive = setActive;
  this.getButtonById = getButtonById;
  this.mouseOver = mouseOver;
  this.mouseOut = mouseOut;
  this.click = click;
}

	//----------------------------------------------------------------------------
	// Member function setSelected() for SimButton object.
	//----------------------------------------------------------------------------

	function setSelected(selected)
	{
	  if(!this.active)
	    return;

	  this.selected = selected;

	  var el = getButtonById(this.buttonName);

	  if(!el)
	    return;

	  if(this.selected)
	  {
	    if(this.buttonName == drillName)
	      if(parent.frames[mainFrameName].drillDown)
	      {
	        if(this.imgSelected!=null)
	          el.src = this.imgSelected;
	      }
	      else
	      {
	        if(this.imgSelectedOff!=null)
	          el.src = this.imgSelectedOff;
	      }
	    else
	      if(this.imgSelected!=null)
	        el.src = this.imgSelected;
	  }
	  else
	    if(this.imgEnabled!=null)
	      el.src = this.imgEnabled;
	}

	//----------------------------------------------------------------------------
	// Member function setActive() for SimButton object.
	//----------------------------------------------------------------------------

	function setActive(active)
	{
	  this.active=active;

	  var el = getButtonById(this.buttonName);

	  if(!el)
	    return;

	  if(this.active)
	  {
	    if(this.imgEnabled!=null)
	      el.src = this.imgEnabled;
	  }
	  else
	    if(this.imgDisabled!=null)
	      el.src = this.imgDisabled;

	  this.setSelected(this.selected);
	}

	//----------------------------------------------------------------------------
	// Member function mouseOver() for SimButton object.
	//----------------------------------------------------------------------------

	function mouseOver()
	{
	  if(!this.active)
	    return;

	  var el = getButtonById(this.buttonName);

	  if(!el)
	    return;

	  if(this.imgOnMouseOver!=null && (!this.selected || this.selected && this.buttonName==drillName))
	    el.src = this.imgOnMouseOver;
	}

	//----------------------------------------------------------------------------
	// Member function mouseOut() for SimButton object.
	//----------------------------------------------------------------------------

	function mouseOut()
	{
	  if(!this.active)
	    return;

	  var el = getButtonById(this.buttonName);

	  if(!el)
	    return;

	  if(this.imgEnabled!=null && (!this.selected || this.selected && this.buttonName==drillName))
	    if(this.selected && this.buttonName==drillName)
	      this.setSelected(true);
	    else
	      el.src = this.imgEnabled;
	}

	//----------------------------------------------------------------------------
	// Member function click() for SimButton object.
	//----------------------------------------------------------------------------

	function click()
	{
	  if(!this.active)
	    return;

	  if((!this.selected || this.selected && this.buttonName==drillName) && this.script!=null && this.script!="")
	  {
	    eval(this.script);
	    this.setSelected(true);
	  }
	}

	//----------------------------------------------------------------------------
	// Member function getButtonById(). Return found element with id elementId
	// if it exists into frame menuFrameName, else - false/null.
	//----------------------------------------------------------------------------

	function getButtonById(elementId)
	{
	  var el = parent.frames[menuFrameName].document.getElementById(elementId);

	  return el;
	}

//----------------------------------------------------------------------------
// Global functions.
//----------------------------------------------------------------------------

	//----------------------------------------------------------------------------
	// Function simMouseOver() for all SimButton objects.
	// Execute mouseOver() function for SimButton object, which is associated by given HTML element.
	//----------------------------------------------------------------------------

	function simMouseOver(element)
	{
	  var button = getSimButton(element);

	  if(button)
	    button.mouseOver();
	}

	//----------------------------------------------------------------------------
	// Function simMouseOut() for all SimButton objects.
	// Execute moudeOut() function for SimButton object, which is associated by given HTML element.
	//----------------------------------------------------------------------------

	function simMouseOut(element)
	{
	  var button = getSimButton(element);

	  if(button)
	    button.mouseOut();
	}

	//----------------------------------------------------------------------------
	// Function simClick() for all SimButton objects.
	// Execute click() function for SimButton object, which is associated by given HTML element.
	//----------------------------------------------------------------------------

	function simClick(element)
	{
	  var button = getSimButton(element);

	  if(button)
	    button.click();
	}

	//----------------------------------------------------------------------------
	// Function getSimButton() for all SimButton objects.
	// Return SimButton object, which is associated by given HTML element.
	//----------------------------------------------------------------------------

	function getSimButton(element)
	{
	  var i;

	  for(i=0; i<simButtons.length; i++)
	    if(element!=null && simButtons[i].buttonName==element.id)
	      return simButtons[i];

	  return null;
	}

//----------------------------------------------------------------------------
// OnClick SimButtons functions
//----------------------------------------------------------------------------

	//----------------------------------------------------------------------------
	// Function simButtonsInit().
	// Sets initial state for all SimButtons objects.
	// Sets simRE event handler functions to "parent" HTML object.
	//----------------------------------------------------------------------------

	function simButtonsInit()
	{
	  parent.onSimulationBegin = onSimulationBegin;
	  parent.onSimulationEnd = onSimulationEnd;
	  parent.onStepBegin = onStepBegin;
	  parent.onStepEnd = onStepEnd;

	  simButtons[homeButton].setActive(true);
	  simButtons[selectButton].setActive(true);
	  simButtons[playButton].setActive(false);
	  simButtons[stopButton].setActive(false);
	  simButtons[skipButton].setActive(false);
	  simButtons[nextStepButton].setActive(false);
	  simButtons[rewindButton].setActive(false);
	  simButtons[drillDownButton].setActive(false);
	  simButtons[setDelayButton].setActive(false);

	  var el = parent.frames[menuFrameName].document.getElementById("delays");
	  if(el)
	    el.disabled=true;
	}

	//----------------------------------------------------------------------------
	// Function simButtonsShow().
	// Show only necessary buttons into menu.
	//----------------------------------------------------------------------------

	function simButtonsShow(playGroup, stopGroup, skipGroup, stepGroup, rewGroup, drilGroup, delayGroup)
	{
	  var buttonsText = '';

	  buttonsText = buttonsText +   '<p>&nbsp;</p>';
	  buttonsText = buttonsText +   '<p><a href="#"> <img id="menu"   src="images/Home.gif"           onClick="simClick(this);" onMouseover="simMouseOver(this);" onMouseout="simMouseOut(this);" width="70" height="16" border="0"></a></p>';
	  buttonsText = buttonsText +   '<p><a href="#"> <img id="selSim" src="images/SelectClick.gif"    onClick="simClick(this);" onMouseover="simMouseOver(this);" onMouseout="simMouseOut(this);" width="70" height="16" border="0"></a></p>';

	  if(playGroup)
	    buttonsText = buttonsText + '<p><a href="#"> <img id="play"   src="images/playButtonDis.gif"  onClick="simClick(this);" onMouseover="simMouseOver(this);" onMouseout="simMouseOut(this);" width="70" height="16" border="0"></a></p>';
	  if(stopGroup)
	    buttonsText = buttonsText + '<p><a href="#"> <img id="stop"   src="images/stopButtonDis.gif"  onClick="simClick(this);" onMouseover="simMouseOver(this);" onMouseout="simMouseOut(this);" width="70" height="16" border="0"></a></p>';
	  if(skipGroup)
	    buttonsText = buttonsText + '<p><a href="#"> <img id="skip"   src="images/skipButtonDis.gif"  onClick="simClick(this);" onMouseover="simMouseOver(this);" onMouseout="simMouseOut(this);" width="70" height="16" border="0"></a></p>';
	  if(stepGroup)
	    buttonsText = buttonsText + '<p><a href="#"> <img id="step"   src="images/stepButtonDis.gif"  onClick="simClick(this);" onMouseover="simMouseOver(this);" onMouseout="simMouseOut(this);" width="70" height="16" border="0"></a></p>';
	  if(rewGroup)
	    buttonsText = buttonsText + '<p><a href="#"> <img id="rewind" src="images/rewButtonDis.gif"   onClick="simClick(this);" onMouseover="simMouseOver(this);" onMouseout="simMouseOut(this);" width="70" height="16" border="0"></a></p>';
	  if(drilGroup)
	    buttonsText = buttonsText + '<p><a href="#"> <img id="drill"  src="images/DrillDis.gif"       onClick="simClick(this);" onMouseover="simMouseOver(this);" onMouseout="simMouseOut(this);" width="70" height="16" border="0"></a></p>';
	  if(delayGroup)
	  {
	    buttonsText = buttonsText + '                <img id="delay"  src="images/delayButtonDis.gif"                           onMouseover="simMouseOver(this);" onMouseout="simMouseOut(this);" width="70" height="16" border="0">';
  	    buttonsText = buttonsText + '<select id="delays" onChange="onDelaysChange();">';
	    buttonsText = buttonsText + '  <option>1</option>';
	    buttonsText = buttonsText + '  <option>2</option>';
	    buttonsText = buttonsText + '  <option>3</option>';
	    buttonsText = buttonsText + '  <option>4</option>';
	    buttonsText = buttonsText + '  <option>5</option>';
	    buttonsText = buttonsText + '  <option>10</option>';
	    buttonsText = buttonsText + '  <option>20</option>';
	    buttonsText = buttonsText + '  <option>30</option>';
	    buttonsText = buttonsText + '  <option>40</option>';
	    buttonsText = buttonsText + '  <option>50</option>';
	    buttonsText = buttonsText + '</select> sec.';
	  }

	  buttonsText = buttonsText +   '<p>&nbsp;</p>';

	  top.menuFrame.menuContent.innerHTML = buttonsText;
	}

	//----------------------------------------------------------------------------
	// Function onHomeClick().
	// Perform 'home' operations to current simulation.
	// Sets 'home' states for SimButtons objects.
	//----------------------------------------------------------------------------

	function onHomeClick()
	{
        window.parent.location.replace("index.htm");
	}

	//----------------------------------------------------------------------------
	// Function onSelSimClick().
	// Perform 'selSim' operations to current simulation.
	// Sets 'selSim' states for SimButtons objects.
	//----------------------------------------------------------------------------

	function onSelSimClick()
	{
        parent.frames[mainFrameName].location.replace("selectSimulation.htm");
	  simButtonsInit();
	}

	//----------------------------------------------------------------------------
	// Function onStopClick().
	// Perform 'stop' operations to current simulation.
	// Sets 'stop' states for SimButtons objects.
	//----------------------------------------------------------------------------

	function onStopClick()
	{
	  parent.frames[mainFrameName].auto=false;
	}

	//----------------------------------------------------------------------------
	// Function onSkipClick().
	// Perform 'skip' operation to current simulation.
	//----------------------------------------------------------------------------

	function onSkipClick()
	{
	  parent.frames[mainFrameName].skip();
	}

	//----------------------------------------------------------------------------
	// Function onPlayClick().
	// Perform 'play' operations to current simulation.
	// Sets 'play' states for SimButtons objects.
	//----------------------------------------------------------------------------

	function onPlayClick()
	{
	  parent.frames[mainFrameName].auto=true;
	  parent.frames[mainFrameName].currentStep.executeNext(false);
	}

	//----------------------------------------------------------------------------
	// Function onStepClick().
	// Perform 'step' operations to current simulation.
	// Sets 'step' states for SimButtons objects.
	//----------------------------------------------------------------------------

	function onStepClick()
	{
	  parent.frames[mainFrameName].auto=false;
	  parent.frames[mainFrameName].currentStep.executeNext(false);
	}

	//----------------------------------------------------------------------------
	// Function onRewindClick().
	// Perform 'rewind' operations to current simulation.
	// Sets 'rewind' states for SimButtons objects.
	//----------------------------------------------------------------------------

	function onRewindClick()
	{
	  parent.frames[mainFrameName].init();

	  selectCurrentDelay();
	}

	//----------------------------------------------------------------------------
	// Function onDrillClick().
	// Perform 'drill' operations to current simulation.
	// Sets 'drill' states for SimButtons objects.
	//----------------------------------------------------------------------------

	function onDrillClick()
	{
	  parent.frames[mainFrameName].drillDown = !parent.frames[mainFrameName].drillDown;
	}

	//----------------------------------------------------------------------------
	// Function selectCurrentDelay().
	// Select current stepTimeout value into combobox "delays".
	//----------------------------------------------------------------------------

	function selectCurrentDelay()
	{
	  var el = parent.frames[menuFrameName].document.getElementById("delays");

	  if(!el)
	    return;

	  var i;

	  for(i=0; i<el.options.length; i++)
	    if(el.options[i].text==parent.frames[mainFrameName].stepTimeout/1000)
	      el.options[i].selected = true;
	}

	//----------------------------------------------------------------------------
	// Function onDelaysChange().
	// Sets new value to stepTimeout when value of combobox "delays" is changed.
	//----------------------------------------------------------------------------

	function onDelaysChange()
	{
	  var el = parent.frames[menuFrameName].document.getElementById("delays");

	  if(!el)
	    return;

	  parent.frames[mainFrameName].setStepTimeout(1000*el.options[el.selectedIndex].text);
	}

//----------------------------------------------------------------------------
// SimRE event handling functions
//----------------------------------------------------------------------------

	//----------------------------------------------------------------------------
	// Function onSimulationBegin().
	// Executed when simRE generate onSimulationBegin event.
	//----------------------------------------------------------------------------

	function onSimulationBegin()
	{
	  simButtons[playButton].setActive(true);
	  simButtons[stopButton].setActive(true);
	  if(parent.frames[mainFrameName].skipEnabled)
	    simButtons[skipButton].setActive(true);
	  else
	    simButtons[skipButton].setActive(false);
	  simButtons[nextStepButton].setActive(true);
	  simButtons[rewindButton].setActive(true);
	  simButtons[drillDownButton].setActive(true);
	  simButtons[setDelayButton].setActive(true);

	  simButtons[selectButton].setSelected(false);
	  simButtons[playButton].setSelected(false);
	  simButtons[stopButton].setSelected(false);
	  simButtons[skipButton].setSelected(false);
	  simButtons[nextStepButton].setSelected(false);
	  simButtons[rewindButton].setSelected(false);
	  simButtons[drillDownButton].setSelected(true);

	  var el = parent.frames[menuFrameName].document.getElementById("delays");
	  if(el)
	    el.disabled=false;

	  selectCurrentDelay();
	}

	//----------------------------------------------------------------------------
	// Function onSimulationEnd().
	// Executed when simRE generate onSimulationEnd event.
	//----------------------------------------------------------------------------

	function onSimulationEnd()
	{
	  simButtons[playButton].setActive(false);
	  simButtons[stopButton].setActive(false);
	  simButtons[skipButton].setActive(false);
	  simButtons[nextStepButton].setActive(false);
	  simButtons[rewindButton].setActive(true);
	  simButtons[drillDownButton].setActive(false);
	  simButtons[setDelayButton].setActive(false);

	  var el = parent.frames[menuFrameName].document.getElementById("delays");
	  if(el)
	    el.disabled=true;
	}

	//----------------------------------------------------------------------------
	// Function onStepBegin().
	// Executed when simRE generate onStepBegin event.
	//----------------------------------------------------------------------------

	function onStepBegin()
	{
	  simButtons[playButton].setActive(true);
	  simButtons[stopButton].setActive(true);
	  if(parent.frames[mainFrameName].skipEnabled)
	    simButtons[skipButton].setActive(true);
	  else
	    simButtons[skipButton].setActive(false);
	  simButtons[nextStepButton].setActive(false);
	  simButtons[rewindButton].setActive(false);

	  simButtons[playButton].setSelected(true);
	  simButtons[stopButton].setSelected(false);
	  simButtons[skipButton].setSelected(false);
	}

	//----------------------------------------------------------------------------
	// Function onStepEnd().
	// Executed when simRE generate onStepEnd event.
	//----------------------------------------------------------------------------

	function onStepEnd()
	{
	  simButtons[playButton].setActive(true);
	  simButtons[stopButton].setActive(false);
	  if(parent.frames[mainFrameName].skipEnabled)
	    simButtons[skipButton].setActive(true);
	  else
	    simButtons[skipButton].setActive(false);
	  simButtons[nextStepButton].setActive(true);
	  simButtons[rewindButton].setActive(true);

	  simButtons[playButton].setSelected(false);
	  simButtons[skipButton].setSelected(false);
	}