/******************************************************************************
* simRE.js                                                                    *
*                                                                             *
* Copyright 2003-2004 by ISY Intellect.                                       *
*                                                                             *
* Functions for Sim Runtime Engine.                                           *
******************************************************************************/

var stepTimeout = 5000;           		// Time before next step begin execution of first operation.
var drillDown = true;             		// Show or no submodels of simulation.
var skipEnabled = true;             	// Is skip button enabled.
var skipClicked = false;             	// Is skip button clicked or not - used from breakpoint method.
var auto = true;                  		// Automatic or no play of simulation.
var stopped = false;              		// Is the simulation currently stopped or running?
var steps = new Array();            	// array of Step objects - must be defined
var timerIds = new Object();              // This object keeps all runinig (and finished) timeouts (by setTimeout function).
							// Used On Click to stop execution of timeout functions.
var stopSimulation = false;			// Execute or no next step into operation (method) body?

var currentModel;                         // Current model object.
var currentModel_firstStepId;             // Current model object.
var currentStep = null;                   // Current step object from steps array.
var standartGUIPage = "about:blank";      // standart GUI page loaded into GUI pane.
var guiFrameName;                         // Name of GUI frame - must be set on model shown.
var lastOverlibObjectName = "";           // last object that show overlib popup.

var drillDownButtonActive = true; // Is the drillDown button accessible?
var delayButtonActive = true;     // Are the delay's objects group accessible?
var playButtonActive = true;      // Is the play button accessible?
var stopButtonActive = true;      // Is the stop button accessible?
var nextButtonActive = true;      // Is the nextStep button accessible?
var skipButtonActive = true;      // Is the skip button accessible?
var rewindButtonActive = true;    // Is the rewind button accessible?

// arrow points vars
var arrowPointUp = '#define x_width 16\n#define x_height 11\nstatic char x_bits[] = {\n' +
' 0x7f,0xfe,  0x7f, 0xfe,  0x3f,0xfc, 0x3f,0xfc,  0x1f,0xf8,  0x1f,0xf8, 0x0f,0xf0,  0x0f,0xf0, 0x07,0xe0,  0x07, 0xe0, 0x33, 0xcc};\n';

var arrowPointLeft = '#define x_width 11\n#define x_height 16\nstatic char x_bits[] = {\n' +
' 0xff,0x7,  0xff,0x7,  0xff,0x3,  0xff,0x0,  0x3f,0x4,  0x0f,0x4,  0x03,0x0,  0x00,0x0,  0x00,0x0,  0x03,0x0,  0x0f,0x4, 0x3f,0x4,  0xff,0x0,  0xff,0x3,  0xff,0x7,  0xff,0x7};\n';

var arrowPoints = new Array( new ArrowPoint(1, 11, 5, "javascript:parent.arrowPointLeft", "javascript:parent.arrowPointUp"), 
//                             new ArrowPoint(3, 11, 5, "..\\images\\arrow_left.gif", "..\\images\\arrow_up.gif"),
                             new ArrowPoint(3, 16, 8, "javascript:parent.arrowPointLeft", "javascript:parent.arrowPointUp"),
                             new ArrowPoint(5, 16, 11, "javascript:parent.arrowPointLeft", "javascript:parent.arrowPointUp"));

var drawTypeNormal = "draw";              // Constant. Used on draw arrow objects.
var drawTypeReverse = "drawReverse";      // Constant. Used on draw arrow objects.
var linePointTimeout = 50;                // Constant. Used on draw arrow objects.
var revealTransTime = 2.0;                // Constant. Time hide and show model's background objects with RevealTrans filter.
var transitionProcessHide = 12;           // Constant. Type of RevealTrans filter transition when hide process.
var transitionProcessShow = 12;           // Constant. Type of RevealTrans filter transition when show process.
var transitionArchitectureHide = 12;      // Constant. Type of RevealTrans filter transition when hide architecture.
var transitionArchitectureShow = 12;      // Constant. Type of RevealTrans filter transition when show architecture.

var NO_LAST_METHOD = "NO_LAST_METHOD";    // Constant. Determinate that no visibility method is used to object.
var SHOW_METHOD = "SHOW";                 // Constant. Determinate that show visibility method is last used to object.
var SHOW_BLINK_METHOD = "SHOW_BLINK";     // Constant. Determinate that showBlink visibility method is last used to object.
var SHOW_TIME_BLINK_METHOD = "SHOW_TIME_BLINK";// Constant. Determinate that showTimeBlink visibility method is last used to object.
var STOP_METHOD = "STOP";                 // Constant. Determinate that stop visibility method is last used to object.
var HIDE_METHOD = "HIDE";                 // Constant. Determinate that hide visibility method is last used to object.
var DRAW_METHOD = "DRAW";                 // Constant. Determinate that draw visibility method is last used to object.
var DRAW_REVERSE_METHOD = "DRAW_REVERSE"; // Constant. Determinate that drawReverse visibility method is last used to object.

var NEW_WINDOW = "new window"             // Constant. Used from GUI objects.
var H_ALIGN_LEFT = "LEFT";                // Constant. Used from GUI objects.
var H_ALIGN_CENTER = "CENTER";            // Constant. Used from GUI objects.
var H_ALIGN_RIGHT = "RIGHT";              // Constant. Used from GUI objects.
var V_ALIGN_TOP = "TOP";                  // Constant. Used from GUI objects.
var V_ALIGN_MIDDLE = "MIDDLE";            // Constant. Used from GUI objects.
var V_ALIGN_BOTTOM = "BOTTOM";            // Constant. Used from GUI objects.

//----------------------------------------------------------------------------
// Constructor for PaneBackground object.
//----------------------------------------------------------------------------

function Model(processFrameName, architectureFrameName, guiFrameName, objects, 
               processFrameWidth, processFrameHeight, 
               architectureFrameWidth, architectureFrameHeight, processFrameRows, architectureFrameCols)
{
  // member attributes

  this.processFrameName = processFrameName;
  this.architectureFrameName = architectureFrameName;
  this.guiFrameName = guiFrameName;
  this.objects = objects;

  this.maxPWidth = 0;
  this.maxPHeight = 0;
  this.maxAWidth = 0;
  this.maxAHeight = 0;

  if(processFrameWidth)
    this.maxPWidth = processFrameWidth;
  if(processFrameHeight)
    this.maxPHeight = processFrameHeight;
  if(architectureFrameWidth)
    this.maxAWidth = architectureFrameWidth;
  if(architectureFrameHeight)
    this.maxAHeight = architectureFrameHeight;

  this.rows = null;
  this.cols = null;

  if(processFrameRows!=null)
  {
    if(processFrameRows=='*')
      this.rows = '*, 0';
    else
      this.rows = '' + processFrameRows + ', *';
  }

  if(architectureFrameCols!=null)
  {
    if(architectureFrameCols=='*')
      this.cols = '*, 0';
    else
      this.cols = '' + architectureFrameCols + ', *';
  }

  this.type = "Model";
  this.name = 'model___' + (new Date() - Math.round(Math.random()*10000));

  // member functions

  this.show = showModel;
  this.hide = hideModel;
  this.remove = removeInner;
  this.initPanesSize = initPanesSize;

  if(this.maxPWidth==0 || this.maxPHeight==0 || this.maxAWidth==0 || this.maxAHeight==0)
    this.initPanesSize();
}

	//----------------------------------------------------------------------------
	// Member function show() for Model object. Show background pictures into 
	// process and architecture panes.
	//----------------------------------------------------------------------------

	function showModel(stepId, guiFrameInitPage)
	{
	  setStepTimeout(stepTimeout); // set correct value of revealTransTime - it depends of stepTimeout.

	  // enable skip button when model is shown
	  skipEnabled = true;
        top.menuFrame.simButtons[top.menuFrame.skipButton].setActive(true);
        skipClicked = false;

        if(stepId!=null)
          currentModel_firstStepId = stepId;
        else
          currentModel_firstStepId = currentStep.stepId;

        guiFrameName = this.guiFrameName;

	  var elP = exists(this.processFrameName, 'root');
	  var elA = exists(this.architectureFrameName, 'root');

        currentModel = this;

	  // if processFrameRows or architectureFrameCols are not given save initial frameset's state.
	  // (else) on next model shown resize the frames to given or to saved.
	  if(this.rows==null || this.cols==null)
	  {
	    this.rows = outerFrameset.rows;
	    this.cols = innerFrameset.cols;
	  }
	  else
	  {
	    outerFrameset.rows = this.rows;
	    innerFrameset.cols = this.cols;
	  }

	  // if overlib is used, hide over object on model's hiding.
	  hideOver();

	  if((elP && elP.innerHTML!="") || (elA && elA.innerHTML!=""))
	  {
	    this.hide();

	    if(guiFrameInitPage==null)
	      if(stepId==null)
	        registerTimeout(this.name, setTimeout("showModelStatic();", revealTransTime*1000));
	      else
	        registerTimeout(this.name, setTimeout("showModelStatic(" + stepId + ");", revealTransTime*1000));
	    else 
	      if(stepId==null)
	        registerTimeout(this.name, setTimeout("showModelStatic(null, '" + guiFrameInitPage + "');", revealTransTime*1000));
	      else
	        registerTimeout(this.name, setTimeout("showModelStatic(" + stepId + ", '" + guiFrameInitPage + "');", revealTransTime*1000));
	  }
	  else
	  {
	    frames[this.guiFrameName].location.replace(standartGUIPage);
	    showModelStatic(stepId, guiFrameInitPage);
	  }
	}

	//----------------------------------------------------------------------------
	// Function showModelStatic(). Show currentModel's background pictures into 
	// process and architecture panes.
	//----------------------------------------------------------------------------

	function showModelStatic(stepId, guiFrameInitPage)
	{
	  var i;

	  var elP = exists(currentModel.processFrameName, 'root');
	  var elA = exists(currentModel.architectureFrameName, 'root');

	  if(elP)
	    elP.style.visibility = "hidden";

	  if(elA)
	    elA.style.visibility = "hidden";

	  elP.style.width = currentModel.maxPWidth;
	  elP.style.height = currentModel.maxPHeight;
	  elA.style.width = currentModel.maxAWidth;
	  elA.style.height = currentModel.maxAHeight;

	  for(i=0; i<currentModel.objects.length; i++)
	    if(currentModel.objects[i].type == "PaneBackground")
	      currentModel.objects[i].show();

	  if(guiFrameInitPage!=null)
	    frames[currentModel.guiFrameName].location.replace(guiFrameInitPage);
	  else
	    frames[currentModel.guiFrameName].location.replace(standartGUIPage);

	  var stopSimulationTemp = stopSimulation;
        stopSimulation = true;

	  for(i=currentStep.currentOpId; i<currentStep.operations.length; i++)
          eval(currentStep.operations[i].operation);

	  if(stepId!=null && steps[stepId]!=null)
	    for(i=1; i<steps[stepId].operations.length; i++)
            eval(steps[stepId].operations[i].operation);

        stopSimulation = stopSimulationTemp;

	  currentStep.currentOpId = currentStep.operations.length; 

	  if(elP)
	  {
	    elP.style.filter = "RevealTrans()";
	    elP.filters.item(0).apply();
	    elP.filters.item(0).transition = transitionProcessShow;
	    elP.style.visibility = "visible";
	    elP.filters.item(0).play(revealTransTime);
	  }

	  if(elA)
	  {
	    elA.style.filter = "RevealTrans()";
	    elA.filters.item(0).apply();
	    elA.filters.item(0).transition = transitionArchitectureShow;
	    elA.style.visibility = "visible";
	    elA.filters.item(0).play(revealTransTime);
	  }

	  currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Member function hide() for Model object. Hide process and architecture panes.
	//----------------------------------------------------------------------------

	function hideModel()
	{
	  var elP = exists(this.processFrameName, 'root');
	  var elA = exists(this.architectureFrameName, 'root');

	  if(elP)
	  {
	    elP.style.filter = "RevealTrans()";
	    elP.filters.item(0).apply();
	    elP.filters.item(0).transition = transitionProcessHide;
	    this.remove(this.processFrameName, 'root');
	    elP.filters.item(0).play(revealTransTime);
	  }
	  else
	    this.remove(this.processFrameName, 'root');

	  if(elA)
	  {
	    elA.style.filter = "RevealTrans()";
	    elA.filters.item(0).apply();
	    elA.filters.item(0).transition = transitionArchitectureHide;
	    this.remove(this.architectureFrameName, 'root');
	    elA.filters.item(0).play(revealTransTime);
	  }
	  else
	    this.remove(this.architectureFrameName, 'root');
	}

	//----------------------------------------------------------------------------
	// Member function remove(). Remove element's entry for element with id 
	// elementId from frame frameName.
	//----------------------------------------------------------------------------

	function removeInner(frameName, elementId)
	{
	  var el = exists(frameName, elementId);

	  if(el!=null)
	    el.innerHTML="";
	}

	//----------------------------------------------------------------------------
	// Member function initPanesSize(). 
	// Init values for size atributes to main model's frames.
	//----------------------------------------------------------------------------

	function initPanesSize()
	{
	  for(i=0; i<this.objects.length; i++)
	    if(this.objects[i].type == "PaneBackground")
	    {
	      if(this.objects[i].frameName == this.processFrameName)
	      {
	        this.maxPWidth = Math.max(this.maxPWidth, this.objects[i].left + this.objects[i].width);
	        this.maxPHeight = Math.max(this.maxPHeight, this.objects[i].top + this.objects[i].height);
	      }

	      if(this.objects[i].frameName == this.architectureFrameName)
	      {
	        this.maxAWidth = Math.max(this.maxAWidth, this.objects[i].left + this.objects[i].width);
	        this.maxAHeight = Math.max(this.maxAHeight, this.objects[i].top + this.objects[i].height);
	      }
	    }
	}

//----------------------------------------------------------------------------
// Constructor for ArrowPoint object. Only for inner use.
//----------------------------------------------------------------------------

function ArrowPoint(forThickness, width, height, srcLeft, srcUp)
{
  // member attributes

  this.forThickness = forThickness;
  this.width = width;
  this.height = height;
  this.srcLeft = srcLeft;
  this.srcUp = srcUp;
}

//----------------------------------------------------------------------------
// Constructor for Hint object.
//----------------------------------------------------------------------------

function Hint()
{
  var i = 0;

  // member attributes

  this.arguments = new Array();

  if(arguments.length==0)
    this.arguments[0]="";
  else
    for(i=0; i<arguments.length; i++)
      this.arguments[i]=arguments[i];

  this.frameAdded = false;  // is frame parameter added to array?
  this.name = 'hint___' + (new Date() - Math.round(Math.random()*10000));

  // member functions

  this.setFrame = setFrame;
}

	//----------------------------------------------------------------------------
	// Member function setFrame() for Hint object. Add parameter FRAME to arguments array.
	//----------------------------------------------------------------------------

	function setFrame()
	{
	  if(this.frameAdded==false)
	  {
          this.arguments[this.arguments.length] = FRAME;
          this.arguments[this.arguments.length] = frames[this.frameName];

	    this.frameAdded = true;
	  }
	  else
          this.arguments[this.arguments.length-1] = frames[this.frameName];
	}

//----------------------------------------------------------------------------
// Constructor for PaneBackground object.
//----------------------------------------------------------------------------

function PaneBackground(frameName, name, width, height, top, left, src, hint, onClick_dbStepId, clickable, autoPlay)
{
  // member attributes

  this.frameName = frameName;
  this.name = name;
  this.width = width;
  this.height = height;
  this.top = top;
  this.left = left;
  this.src = src;

  this.hint = hint;
  if(this.hint!=null)        
    this.hint.frameName = frameName;

  this.onClick_dbStepId = onClick_dbStepId;
  this.clickable = (clickable==null?true:clickable);
  if(this.onClick_dbStepId==null) 
    this.clickable = false;
  this.autoPlay = (autoPlay==null?0:autoPlay);
 
  this.type = "PaneBackground";
  this.uniqueDivName = this.name;

  // member functions

  this.show = showPaneBackground;
  this.mouseEvents = mouseEvents;
  this.setClickable = setClickable;
  this.setClickableStep = setClickableStep;
  this.setHint = setHint;
}

	//----------------------------------------------------------------------------
	// Member function show() for PaneBackground object. Show picture into it's pane.
	//----------------------------------------------------------------------------

	function showPaneBackground()
	{
	  if(!exists(this.frameName, this.uniqueDivName))
	    frames[this.frameName].root.insertAdjacentHTML('BeforeEnd', '<img ' + this.mouseEvents() + ' style="position: absolute; z-index: 0; top: ' + this.top + '; left: ' + this.left + '" src="' + this.src + '" width="' + this.width + '" height="' + this.height + '" id="' + this.uniqueDivName + '">');  

	  this.setClickable(this.clickable, false);
	}

//----------------------------------------------------------------------------
// Constructor for Picture object.
//----------------------------------------------------------------------------

function Picture(frameName, name, width, height, top, left, src, animated, hint, onClick_dbStepId, clickable, autoPlay)
{
  // member attributes

  this.frameName = frameName;
  this.name = name;
  this.width = width;
  this.height = height;
  this.top = top;
  this.left = left;
  this.src = src;
  if(animated==null)
    this.animated = false;
  else
    this.animated = animated;

  this.hint = hint;
  if(this.hint!=null)
    this.hint.frameName = frameName;

  this.onClick_dbStepId = onClick_dbStepId;
  this.clickable = (clickable==null?true:clickable);
  if(this.onClick_dbStepId==null) 
    this.clickable = false;
  this.autoPlay = (autoPlay==null?0:autoPlay);

  this.uniqueDivName = this.name;

  // this object is used to save initial state of some data that can be changed
  // and other special data
  this.data = new Object();
  this.data.top = this.top;
  this.data.left = this.left;
  this.data.clickable = this.clickable;
  this.data.lastMethod = NO_LAST_METHOD; // Determinate which method was last used.


  // member functions

  this.show = show;
  this.showBlink = showBlink;
  this.stop = stop;
  this.showTimeBlink = showTimeBlink;
  this.hide = hide;

  this.create = createPicture;
  this.remove = remove;
  this.isVisible = isVisible;
  this.setPosition = setPosition;
  this.mouseEvents = mouseEvents;
  this.reset = resetPicture;
  this.setClickable = setClickable;
  this.setClickableStep = setClickableStep;
  this.setHint = setHint;
}

	//----------------------------------------------------------------------------
	// Member function create() for Picture object. Create picture.
	//----------------------------------------------------------------------------

	function createPicture()
	{
	  this.uniqueDivName = this.name + (new Date() - 0);

	  if(this.animated)
	    src = this.src + "?" + this.uniqueDivName;
	  else
	    src = this.src;

	  frames[this.frameName].root.insertAdjacentHTML('BeforeEnd', '<img ' + this.mouseEvents() + ' style="position: absolute; z-index: 1; top: ' + this.top + '; left: ' + this.left + '" src="' + src + '" width="' + this.width + '" height="' + this.height + '" id="' + this.uniqueDivName + '">');  

	  this.setClickable(this.clickable, false);
	}

	//----------------------------------------------------------------------------
	// Member function reset() for Picture object. 
	// Restore init values of object's attributes.
	//----------------------------------------------------------------------------

	function resetPicture()
	{
	  this.top = this.data.top;
	  this.left = this.data.left;
	  this.clickable = this.data.clickable;
        this.data.lastMethod=NO_LAST_METHOD;
	}

//----------------------------------------------------------------------------
// Constructor for Highlight
//----------------------------------------------------------------------------

function Highlight(frameName, name, width, height, top, left, src, thickness, hint, onClick_dbStepId, clickable, autoPlay)
{
  // member attributes

  this.frameName = frameName;
  this.name = name;
  this.width = width;
  this.height = height;
  this.top = top;
  this.left = left;
  this.src = src;
  this.thickness = thickness;

  this.hint = hint;
  if(this.hint!=null)
    this.hint.frameName = frameName;

  this.onClick_dbStepId = onClick_dbStepId;
  this.clickable = (clickable==null?true:clickable);
  if(this.onClick_dbStepId==null) 
    this.clickable = false;
  this.autoPlay = (autoPlay==null?0:autoPlay);

  this.uniqueDivName = this.name;

  // this object is used to save initial state of some data that can be changed
  // and other special data
  this.data = new Object();
  this.data.top = this.top;
  this.data.left = this.left;
  this.data.clickable = this.clickable;
  this.data.lastMethod = NO_LAST_METHOD; // Determinate which method was last used.


  // member functions

  this.show = show;
  this.showBlink = showBlink;
  this.stop = stop;
  this.showTimeBlink = showTimeBlink;
  this.hide = hide;

  this.create = createHighlight;
  this.remove = remove;
  this.isVisible = isVisible;
  this.setPosition = setPosition;
  this.mouseEvents = mouseEvents;
  this.reset = resetHighlight;
  this.setClickable = setClickable;
  this.setClickableStep = setClickableStep;
  this.setHint = setHint;
}

	//----------------------------------------------------------------------------
	// Member function create() for Highlight object. Create Highlight.
	//----------------------------------------------------------------------------

	function createHighlight(color)
	{
	  this.uniqueDivName = this.name + (new Date() - 0);

	  frames[this.frameName].root.insertAdjacentHTML('BeforeEnd', '<img ' + this.mouseEvents() + ' style="position: absolute; z-index: 1; top: ' + this.top + '; left: ' + this.left + '; filter:mask(color=' + color + ') glow(color=' + color + ', Strength=' + this.thickness + ')" src="' + this.src + '" width="' + this.width + '" height="' + this.height + '" id="' + this.uniqueDivName + '">');  

	  this.setClickable(this.clickable, false);
	}

	//----------------------------------------------------------------------------
	// Member function reset() for Highlight object. 
	// Restore init values of object's attributes.
	//----------------------------------------------------------------------------

	function resetHighlight()
	{
	  this.top = this.data.top;
	  this.left = this.data.left;
	  this.clickable = this.data.clickable;
        this.data.lastMethod=NO_LAST_METHOD;
	}

//----------------------------------------------------------------------------
// Constructor for Label
//----------------------------------------------------------------------------

function Label(frameName, name, top, left, text, style, width, height, hint, onClick_dbStepId, clickable, autoPlay)
{
  // member attributes

  this.frameName = frameName;
  this.name = name;
  this.top = top;
  this.left = left;
  this.text = text;
  this.style = style;
  this.width = width==null?-1:width;
  this.height = height==null?-1:height;

  this.hint = hint;
  if(this.hint!=null)
    this.hint.frameName = frameName;

  this.onClick_dbStepId = onClick_dbStepId;
  this.clickable = (clickable==null?true:clickable);
  if(this.onClick_dbStepId==null) 
    this.clickable = false;
  this.autoPlay = (autoPlay==null?0:autoPlay);

  this.uniqueDivName = this.name;

  // this object is used to save initial state of some data that can be changed
  // and other special data
  this.data = new Object();
  this.data.top = this.top;
  this.data.left = this.left;
  this.data.text = this.text;
  this.data.width = this.width;
  this.data.height = this.height;
  this.data.clickable = this.clickable;
  this.data.lastMethod = NO_LAST_METHOD; // Determinate which method was last used.


  // member functions

  this.show = show;
  this.showBlink = showBlink;
  this.stop = stop;
  this.showTimeBlink = showTimeBlink;
  this.hide = hide;

  this.create = createLabel;
  this.remove = remove;
  this.isVisible = isVisible;

  this.setText = setText;
  this.setPosition = setPosition;
  this.mouseEvents = mouseEvents;
  this.reset = resetLabel;
  this.setClickable = setClickable;
  this.setClickableStep = setClickableStep;
  this.setHint = setHint;
}

	//----------------------------------------------------------------------------
	// Member function create() for Label object. Show Label into it's pane.
	//----------------------------------------------------------------------------

	function createLabel(color)
	{
	  this.uniqueDivName = this.name + (new Date() - 0);

	  frames[this.frameName].root.insertAdjacentHTML('BeforeEnd', '<div ' + this.mouseEvents() + ' class="' + this.style + '" style="height:' + this.height + '; width:' + this.width + '; overflow:visible; position: absolute; z-index: 3; color: ' + color + '; top: ' + this.top + '; left: ' + this.left + ';" id="' + this.uniqueDivName + '"><table width="100%" height="100%" style="border-collapse: collapse" cellspacing="0" cellpadding="0"><tr><td class="' + this.style + '" style="color:' + color + '">' + this.text + '</td></tr></table></div>');  

	  this.setClickable(this.clickable, false);
	}

	//----------------------------------------------------------------------------
	// Member function setText() for Label object. Change some properties of Label.
	// If object is visible - last visibility operation continue.
	//----------------------------------------------------------------------------

	function setText(text, width, height)
	{ 
	  this.text = text;
	  if(width!=null && width!=0)
	    this.width = width;
	  if(height!=null && height!=0)
	    this.height = height;

	  if(this.isVisible())
        {
	    if(this.data.lastMethod==SHOW_BLINK_METHOD)
	      this.showBlink(this.data.flashTimer, this.data.color);
	    else
	      this.show(this.data.color);
        }
	  else
	    currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Member function reset() for Label object. 
	// Restore init values of object's attributes.
	//----------------------------------------------------------------------------

	function resetLabel()
	{
	  this.top = this.data.top;
	  this.left = this.data.left;
	  this.text = this.data.text;
	  this.width = this.data.width;
	  this.height = this.data.height;
	  this.clickable = this.data.clickable;
        this.data.lastMethod=NO_LAST_METHOD;
	}

//----------------------------------------------------------------------------
// Constructor for GUI
//----------------------------------------------------------------------------

function GUI(url)
{
  // member attributes

  this.frameName = null;
  this.childWindow = null;
  this.url = url;
  this.type = "GUI";

  // member functions

  this.show = showGUI;
  this.hide = hideGUI;
  this.newWindow = newWindow;
}

	//----------------------------------------------------------------------------
	// Member function show() for GUI object. Show GUI into it's frame or new browser window.
	//----------------------------------------------------------------------------

	function showGUI(frameName, width_p, height_p, left_p, top_p, winName_p, status_p, menubar_p, toolbar_p, resizable_p, location_p, scrollbars_p, directories_p, focused_p)
	{
	  this.frameName = frameName;

	  if(this.newWindow())
	  {
	    winName = (winName_p!=null? winName_p: "GUI___" + (new Date() - 0));

	    var width = width_p!=null?width_p:400;
	    var height = height_p!=null?height_p:300;
	    var status = status_p==null?"no":(status_p==true?"yes":"no");
	    var menubar = menubar_p==null?"no":(menubar_p==true?"yes":"no");
	    var toolbar = toolbar_p==null?"no":(toolbar_p==true?"yes":"no");
	    var resizable = resizable_p==null?"yes":(resizable_p==true?"yes":"no");
	    var location = location_p==null?"no":(location_p==true?"yes":"no");
	    var scrollbars = scrollbars_p==null?"no":(scrollbars_p==true?"yes":"no");
	    var directories = directories_p==null?"no":(directories_p==true?"yes":"no");
	    var focused = focused_p==null?true:focused_p;

	    var left = 0;
	    var top = 0;

	    // Set absolute position of new window...
	    if(typeof left_p=='number')
	      left = left_p;

	    if(typeof top_p=='number')
	      top = top_p;

	    // ... or calculate realtive position of new window
	    if(typeof left_p=='string')
	    {
	      // Value of left_p may be number or some of string constants H_ALIGN_LEFT, H_ALIGN_CENTER, H_ALIGN_RIGHT 
	      if(left_p==H_ALIGN_CENTER)
	        left = (screen.availWidth - width)/2;
	      else if(left_p==H_ALIGN_RIGHT)
	        left = screen.availWidth - width;
	    }

	    if(typeof top_p=='string')
	    {
	      // Value of top_p may be number or some of string constants V_ALIGN_TOP, V_ALIGN_MIDDLE, V_ALIGN_BOTTOM 
	      if(top_p==V_ALIGN_MIDDLE)
	        top = (screen.availHeight - height)/2;
	      else if(top_p==V_ALIGN_BOTTOM)
	        top = screen.availHeight - height;
	    }

	    if(left<0)
	      left = 0;

	    if(top<0)
	      top = 0;

          this.childWindow = window.open(this.url, winName, 'directories=' + directories + ', status=' + status + ', menubar=' + menubar + ', toolbar=' + toolbar + ', resizable=' + resizable + ', location=' + location + ', scrollbars=' + scrollbars + ', width=' + width + ', height=' + height + ', left=' + left + ', top=' + top);
          try
          {
            this.childWindow.resizeTo(width, height);
          }
          catch(ex)
          {}
          try
          {
            this.childWindow.moveTo(left, top);
          }
          catch(ex)
          {}
          if(focused)
            this.childWindow.focus();
	  }
	  else
	    frames[this.frameName].location.replace(this.url);

        currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Member function hide() for GUI object. Hide GUI.
	//----------------------------------------------------------------------------

	function hideGUI()
	{
	  if(this.newWindow())
	    this.childWindow.close();
	  else
	    if(frames[this.frameName])
	      frames[this.frameName].location.replace(standartGUIPage);

        currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Member function newWindow() for GUI object. 
	// Return true if GUI object must be placed to new window, else - false.
	//----------------------------------------------------------------------------

	function newWindow()
	{
	  return this.frameName==NEW_WINDOW?true:false;
	}

//----------------------------------------------------------------------------
// Constructor for Delta
//----------------------------------------------------------------------------

function Delta(dx, dy)
{
  // member attributes

  this.dx = dx;
  this.dy = dy;
}

//----------------------------------------------------------------------------
// Constructor for Section
//----------------------------------------------------------------------------

function Section(top, left, dx, dy)
{
  // member attributes

  this.top = top;
  this.left = left;
  this.dx = dx;
  this.dy = dy;

  // member functions

  this.show = showSection;
  this.draw = drawSection;

  this.remove = remove;
}

	//----------------------------------------------------------------------------
	// Member function show() for Section object. Show Section into it's pane.
	// type = 0 - section without arrow Points
	// type = 1 - section with first arrow Point
	// type = 2 - section with last arrow Point
	// type = 3 - section with first and last arrow Points
	//----------------------------------------------------------------------------

	function showSection(frameName, arrowName, color, thickness, type)
	{
	  var arrowId = exists(frameName, arrowName);

	  if(!arrowId)
	    return;

	  var t = this.top;
	  var l = this.left;
	  var arrowPoint = getArrowPoint(thickness);
	  var w = arrowPoint.height;
	  var len = Math.abs(this.dx + this.dy);

	  showArrowPoint(frameName, arrowName, color, thickness, this.top, this.left, this.dx, this.dy, type);

	  if(type==1 || type==3)
	  {
	    len = len - w;

	    if(this.dx!=0)
	    {
	      t = this.top;

	      if(this.dx>0)  // first left oriented point
	        l = this.left + w;
	    }
	    else
	    {
	      l = this.left;

	      if(this.dy>0)  // first up oriented point
	        t = this.top + w;
	    }
	  }

	  if(type==2 || type==3)
	  {
	    len = len - w;

	    if(this.dx!=0)
	    {
	      t = this.top;

	      if(this.dx<0)
	        l = this.left + w;
	    }
	    else
	    {
	      l = this.left;

	      if(this.dy<0)
	        t = this.top + w;
	    }
	  }

	  if(len<0)
	    len=0;

	  if(this.dx!=0)
	    arrowId.insertAdjacentHTML('BeforeEnd', '<div style="position: absolute; z-index: 2; top: ' + t + 'px; left: ' + l + 'px; width: ' + len + 'px; height: ' + thickness + 'px; background-color: ' + color + '"><div/></div>');
	  else
	    arrowId.insertAdjacentHTML('BeforeEnd', '<div style="position: absolute; z-index: 2; top: ' + t + 'px; left: ' + l + 'px; width: ' + thickness + 'px; height: ' + len + 'px; background-color: ' + color + '"><div/></div>');
	}

	//----------------------------------------------------------------------------
	// Function showArrowPoint(). Show arrows's point(s) into it's (their's) pane(s).
	// what==0 - show all Arrow Points if they exists.
	// what==1 - show only first Arrow Point if exists.
	// what==2 - show only last Arrow Point if exists.
	//----------------------------------------------------------------------------

	function showArrowPoint(frameName, arrowName, color, thickness, top, left, dx, dy, type, what)
	{
	  if(type==0)
	    return;

	  if(what==null)
	    what = 0;

	  var arrowId = exists(frameName, arrowName);

	  if(!arrowId)
	    return;

	  var t, l, w, h, pointSrc;
	  var filter = "mask(color='" + color + "')";
	  var arrowPoint = getArrowPoint(thickness);

	  if((type==1 || type==3) && what!=2)
	  {
	    if(dx!=0)
	    {
	      pointSrc = arrowPoint.srcLeft;
	      w = arrowPoint.height;
	      h = arrowPoint.width;
	      t = top - Math.floor((h-thickness)/2);

	      if(dx>0)  // first left oriented point
	      {
	        l = left;
	      }
	      else      // first right oriented point
	      {
	        l = left - dx - w;
	        filter = filter + " flipH";
	      }
	    }
	    else
	    {
	      pointSrc = arrowPoint.srcUp;
	      w = arrowPoint.width;
	      h = arrowPoint.height;
	      l = left - Math.floor((w-thickness)/2);

	      if(dy>0)  // first up oriented point
	      {
	        t = top;
	      }
	      else      // first down oriented point
	      {
	        t = top - dy - h;
	        filter = filter + " flipV";
	      }	      
	    }
	    arrowId.insertAdjacentHTML('BeforeEnd', '<img style="position: absolute; z-index: 2; filter: ' +  filter + '; top: ' + t + 'px; left: ' + l + 'px; width: ' + w + 'px; height: ' + h + 'px;" src="' + pointSrc + '"></img>');
	  }

	  filter = "mask(color='" + color + "')";

	  if((type==2 || type==3) && what!=1)
	  {
	    if(dx!=0)
	    {
	      pointSrc = arrowPoint.srcLeft;
	      w = arrowPoint.height;
	      h = arrowPoint.width;
	      t = top - Math.floor((h-thickness)/2);

	      if(dx>0)
	      {
	        l = left + dx - w;
	        filter = filter + " flipH"; 
	      }
	      else
	      {
	        l = left;
	      }
	    }
	    else
	    {
	      pointSrc = arrowPoint.srcUp;
	      w = arrowPoint.width;
	      h = arrowPoint.height;
	      l = left - Math.floor((w-thickness)/2);

	      if(dy>0)
	      {
	        t = top + dy - h;
	        filter = filter + " flipV"; 
	      }
	      else
	      {
	        t = top;
	      }	      
	    }
	    arrowId.insertAdjacentHTML('BeforeEnd', '<img style="position: absolute; z-index: 2; filter: ' +  filter + '; top: ' + t + 'px; left: ' + l + 'px; width: ' + w + 'px; height: ' + h + 'px;" src="' + pointSrc + '"></img>');
	  }
	}

	//----------------------------------------------------------------------------
	// Member function draw() for Section object. Draw Section into it's pane.
	// Parameter type must be "drawReverse" or "draw".
	// Parameter drawingStep gives a step by which arrow must be increased.
	// Parameter currentPos is position of section object into Arrow's array.
	// Other params are given from arrow object.
	//----------------------------------------------------------------------------

	function drawSection(frameName, arrowName, color, thickness, first, last, drawingStep, wait, currentPos, drawType)
	{
	  eval("var arrow = " + arrowName + ";");

	  var sectionName = arrow.uniqueDivName + "___" + currentPos;

	  var arrowId = exists(frameName, arrow.uniqueDivName); 

	  if(!arrowId)
	    return;

	  var section;
	  var type;

	  if(first && currentPos==0 && drawType==drawTypeNormal)
        {
	    section = arrow.sections[0];
	    type = getSectionType(arrow.sections.length, 0, first, last);
	    showArrowPoint(frameName, arrow.uniqueDivName, color, thickness, section.top, section.left, section.dx, section.dy, type, 1);
	  }

	  if(last && currentPos==arrow.sections.length-1 && drawType==drawTypeReverse)
        {
	    section = arrow.sections[arrow.sections.length-1];
	    type = getSectionType(arrow.sections.length, arrow.sections.length-1, first, last);
	    showArrowPoint(frameName, arrow.uniqueDivName, color, thickness, section.top, section.left, section.dx, section.dy, type, 2);
	  }

	  if(currentPos<0 || currentPos>arrow.sections.length-1)
        {

	    if(first && drawType==drawTypeReverse)
	    {
	      section = arrow.sections[0];
	      type = getSectionType(arrow.sections.length, 0, first, last);
	      showArrowPoint(frameName, arrow.uniqueDivName, color, thickness, section.top, section.left, section.dx, section.dy, type, 1);
	    }

	    if(last && drawType==drawTypeNormal)
	    {
	      section = arrow.sections[arrow.sections.length-1];
	      type = getSectionType(arrow.sections.length, arrow.sections.length-1, first, last);
	      showArrowPoint(frameName, arrow.uniqueDivName, color, thickness, section.top, section.left, section.dx, section.dy, type, 2);
	    }

	    if(wait)
            currentStep.executeNext(true, true);

          return;
        }

	  section = arrow.sections[currentPos]; 

	  var arrowPoint = getArrowPoint(thickness);

	  var w = arrowPoint.height;
	  var width = section.dy!=0 ? thickness : 0;
	  var height = section.dx!=0 ? thickness : 0;
	  var len = Math.abs(section.dx + section.dy);
	  var t = section.top;
	  var l = section.left;

	  type = getSectionType(arrow.sections.length, currentPos, first, last);

	  if(type==0 || type==2)
	  {
	    if(section.dx!=0)
	    {
	      if(!(section.dx>0 && drawType==drawTypeNormal || section.dx<0 && drawType==drawTypeReverse))
	        l = section.left + len;

	      if(type==2 && drawType==drawTypeReverse)
	        if(section.dx>0)
	          l = l - w;
	        else
	          l = l + w;
	    }
	    else
	    {
	      if(!(section.dy>0 && drawType==drawTypeNormal || section.dy<0 && drawType==drawTypeReverse))
	        t = section.top + len;

	      if(type==2 && drawType==drawTypeReverse)
	        if(section.dy>0)
	          t = t - w;
	        else
	          t = t + w;
	    }

	    if(type==2)
	      len = len - w;
	  }

	  if(type==1 || type==3)
	  {
	    len = len - w;

	    if(section.dx!=0)
	    {
	      if(section.dx>0 && drawType==drawTypeNormal || section.dx<0 && drawType==drawTypeReverse)
	        l = section.left + w;
	      else
	        l = section.left + len;

	      if(type==1 && drawType==drawTypeReverse)
	        if(section.dx>0)
	          l = l + w;
	        else
	          l = l - w;
	    }
	    else
	    {
	      if(section.dy>0 && drawType==drawTypeNormal || section.dy<0 && drawType==drawTypeReverse)
	        t = section.top + w;
	      else
	        t = section.top + len;

	      if(type==1 && drawType==drawTypeReverse)
	        if(section.dy>0)
	          t = t + w;
	        else
	          t = t - w;
	    }

	    if(type==3)
	      len = len - w;
	  }

	  arrowId.insertAdjacentHTML('BeforeEnd', '<div style="position: absolute; z-index: 2; top: ' + t + 'px; left: ' + l + 'px; width: ' + width + 'px; height: ' + height + 'px; background-color: ' + color + '" id="' + sectionName + '"><div/></div>');

	  if(section.dx!=0)
	    if(section.dx>0 && drawType==drawTypeNormal || section.dx<0 && drawType==drawTypeReverse)
	      lineLeftToRight(frameName, arrowName, color, thickness, first, last, drawingStep, wait, currentPos, drawType, sectionName, len);
	    else
	      lineRightToLeft(frameName, arrowName, color, thickness, first, last, drawingStep, wait, currentPos, drawType, sectionName, len);
	  else
	    if(section.dy>0 && drawType==drawTypeNormal || section.dy<0 && drawType==drawTypeReverse)
	      lineUpToBottom(frameName, arrowName, color, thickness, first, last, drawingStep, wait, currentPos, drawType, sectionName, len);
	    else
	      lineBottomToUp(frameName, arrowName, color, thickness, first, last, drawingStep, wait, currentPos, drawType, sectionName, len);
	}

	//----------------------------------------------------------------------------
	// Function lineLeftToRight() for Section object. Draw line.
	//----------------------------------------------------------------------------

	function lineLeftToRight(frameName, arrowName, color, thickness, first, last, drawingStep, wait, currentPos, drawType, lineId, length, initStep)
	{
	  var el = exists(frameName, lineId);

	  if(el==null)
	    return;

	  eval("var arrow = " + arrowName + ";");
	  var arrowId = exists(frameName, arrow.uniqueDivName); 

	  if(!arrowId)
	    return;

 	  var realStep = drawingStep;

	  if(el.style.pixelWidth + realStep > length)
	    realStep = length - el.style.pixelWidth;

	  el.style.pixelWidth += realStep;
	  if(el.style.pixelWidth<length)
	    registerTimeout(arrowName, setTimeout("lineLeftToRight('" + frameName + "', '" + arrowName + "', '" + color + "', " + thickness + ", " + first + ", " + last + ", " + drawingStep + ", " + wait + ", " + currentPos + ", '" + drawType + "', '" + lineId + "', " + length + ");", linePointTimeout));
	  else
	    registerTimeout(arrowName, setTimeout("drawSection('" + frameName + "', '" + arrowName + "', '" + color + "', " + thickness + ", " + first + ", " + last + ", " + drawingStep + ", " + wait + ", " + (drawType==drawTypeNormal?currentPos+1:currentPos-1) + ", '" + drawType + "');", linePointTimeout));
	}

	//----------------------------------------------------------------------------
	// Function lineRightToLeft() for Section object. Draw line.
	//----------------------------------------------------------------------------

	function lineRightToLeft(frameName, arrowName, color, thickness, first, last, drawingStep, wait, currentPos, drawType, lineId, length, initStep)
	{
	  var el = exists(frameName, lineId);

	  if(el==null)
	    return;

	  eval("var arrow = " + arrowName + ";");
	  var arrowId = exists(frameName, arrow.uniqueDivName); 

	  if(!arrowId)
	    return;

 	  var realStep = drawingStep;

	  if(el.style.pixelWidth + realStep > length)
	    realStep = length - el.style.pixelWidth;

	  el.style.pixelWidth += realStep;
	  el.style.pixelLeft -= realStep;

	  if(el.style.pixelWidth<length)
	    registerTimeout(arrowName, setTimeout("lineRightToLeft('" + frameName + "', '" + arrowName + "', '" + color + "', " + thickness + ", " + first + ", " + last + ", " + drawingStep + ", " + wait + ", " + currentPos + ", '" + drawType + "', '" + lineId + "', " + length + ");", linePointTimeout));
	  else
	    registerTimeout(arrowName, setTimeout("drawSection('" + frameName + "', '" + arrowName + "', '" + color + "', " + thickness + ", " + first + ", " + last + ", " + drawingStep + ", " + wait + ", " + (drawType==drawTypeNormal?currentPos+1:currentPos-1) + ", '" + drawType + "');", linePointTimeout));
	}

	//----------------------------------------------------------------------------
	// Function lineUpToBottom() for Section object. Draw line.
	//----------------------------------------------------------------------------

	function lineUpToBottom(frameName, arrowName, color, thickness, first, last, drawingStep, wait, currentPos, drawType, lineId, length, initStep)
	{
	  var el = exists(frameName, lineId);

	  if(el==null)
	    return;

	  eval("var arrow = " + arrowName + ";");
	  var arrowId = exists(frameName, arrow.uniqueDivName); 

	  if(!arrowId)
	    return;

 	  var realStep = drawingStep;

	  if(el.style.pixelHeight + realStep > length)
	    realStep = length - el.style.pixelHeight;

	  el.style.pixelHeight += realStep;

	  if(el.style.pixelHeight<length)
	    registerTimeout(arrowName, setTimeout("lineUpToBottom('" + frameName + "', '" + arrowName + "', '" + color + "', " + thickness + ", " + first + ", " + last + ", " + drawingStep + ", " + wait + ", " + currentPos + ", '" + drawType + "', '" + lineId + "', " + length + ");", linePointTimeout));
	  else
	    registerTimeout(arrowName, setTimeout("drawSection('" + frameName + "', '" + arrowName + "', '" + color + "', " + thickness + ", " + first + ", " + last + ", " + drawingStep + ", " + wait + ", " + (drawType==drawTypeNormal?currentPos+1:currentPos-1) + ", '" + drawType + "');", linePointTimeout));
	}

	//----------------------------------------------------------------------------
	// Function lineBottomToUp() for Section object. Draw line.
	//----------------------------------------------------------------------------

	function lineBottomToUp(frameName, arrowName, color, thickness, first, last, drawingStep, wait, currentPos, drawType, lineId, length, initStep)
	{
	  var el = exists(frameName, lineId);

	  if(el==null)
	    return;

	  eval("var arrow = " + arrowName + ";");
	  var arrowId = exists(frameName, arrow.uniqueDivName); 

	  if(!arrowId)
	    return;

 	  var realStep = drawingStep;

	  if(el.style.pixelHeight + realStep > length)
	    realStep = length - el.style.pixelHeight;

	  el.style.pixelHeight += realStep;
	  el.style.pixelTop -= realStep;

	  if(el.style.pixelHeight<length)
	    registerTimeout(arrowName, setTimeout("lineBottomToUp('" + frameName + "', '" + arrowName + "', '" + color + "', " + thickness + ", " + first + ", " + last + ", " + drawingStep + ", " + wait + ", " + currentPos + ", '" + drawType + "', '" + lineId + "', " + length + ");", linePointTimeout));
	  else
	    registerTimeout(arrowName, setTimeout("drawSection('" + frameName + "', '" + arrowName + "', '" + color + "', " + thickness + ", " + first + ", " + last + ", " + drawingStep + ", " + wait + ", " + (drawType==drawTypeNormal?currentPos+1:currentPos-1) + ", '" + drawType + "');", linePointTimeout));
	}

	//----------------------------------------------------------------------------
	// function getSectionType() for Section object. Return type of section.
	// type = 0 - section without arrow Points
	// type = 1 - section with first arrow Point
	// type = 2 - section with last arrow Point
	// type = 3 - section with first and last arrow Points
	//----------------------------------------------------------------------------

	function getSectionType(sectionsLength, sectionPos, first, last)
	{
	  if(sectionsLength==1 && sectionPos==0 && first && last)
          return 3;

	  if(sectionsLength>0 && sectionPos==0 && first)
          return 1;

	  if(sectionsLength>0 && sectionPos==sectionsLength-1 && last)
          return 2;

        return 0;
	}

//----------------------------------------------------------------------------
// Constructor for Arrow
//----------------------------------------------------------------------------

function Arrow(frameName, name, sections, thickness, x, y, hint, onClick_dbStepId, clickable, autoPlay)
{
  // member attributes

  this.frameName = frameName;
  this.name = name;
  this.thickness = thickness;
  this.x = x;
  this.y = y;

  this.hint = hint;
  if(this.hint!=null)
    this.hint.frameName = frameName;

  this.onClick_dbStepId = onClick_dbStepId;
  this.clickable = (clickable==null?true:clickable);
  if(this.onClick_dbStepId==null) 
    this.clickable = false;
  this.autoPlay = (autoPlay==null?0:autoPlay);

  this.sections = computeSections(sections, x, y, thickness);
  this.length = computeLength(this.sections);

  this.uniqueDivName = this.name;

  // this object is used to save initial state of some data that can be changed
  // and other special data
  this.data = new Object();
  this.data.sections = sections;
  this.data.x = this.x; 
  this.data.y = this.y;
  this.data.clickable = this.clickable;
  this.data.lastMethod = NO_LAST_METHOD; // Determinate which method was last used.


  // member functions

  this.show = show;
  this.showBlink = showBlink;
  this.stop = stop;
  this.showTimeBlink = showTimeBlink;
  this.draw = drawArrow;
  this.drawReverse = drawReverseArrow;
  this.hide = hide;

  this.create = createArrow;
  this.remove = remove;
  this.isVisible = isVisible;
  this.setPosition = setPosition;
  this.mouseEvents = mouseEvents;
  this.reset = resetArrow;
  this.setClickable = setClickable;
  this.setClickableStep = setClickableStep;
  this.setHint = setHint;
}

	//----------------------------------------------------------------------------
	// Member function draw() for Arrow object. Draw Arrow progressively 
	// into it's pane.
	//----------------------------------------------------------------------------

	function drawArrow(color, first, last, wait, drawTime)
	{
        this.data.lastMethod = DRAW_METHOD;
        this.data.color = color;
        this.data.first = first;
        this.data.last = last;
        this.data.wait = wait;
        this.data.drawTime = drawTime;

	  if(exists(this.frameName, this.uniqueDivName))
	    this.remove();

	  this.uniqueDivName = this.name + (new Date() - 0);

	  frames[this.frameName].root.insertAdjacentHTML('BeforeEnd', '<div ' + this.mouseEvents() + ' style="position: absolute; z-index: 2;" id="' + this.uniqueDivName + '"></div>');  

	  this.setClickable(this.clickable, false);

	  var arrowPoint = getArrowPoint(this.thickness);
	  var drawingLength = this.length - (first?arrowPoint.height:0) - (last?arrowPoint.height:0);

	  var drawingStep = Math.max(1, Math.floor(drawingLength*linePointTimeout/drawTime));

        if(this.sections.length>0)
          this.sections[0].draw(this.frameName, this.name, color, this.thickness, first, last, drawingStep, wait, 0, drawTypeNormal);

        if(!wait)
          currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Member function drawReverse() for Arrow object. Draw reverse Arrow 
	// progressively into it's pane.
	//----------------------------------------------------------------------------

	function drawReverseArrow(color, first, last, wait, drawTime)
	{
        this.data.lastMethod = DRAW_REVERSE_METHOD;
        this.data.color = color;
        this.data.first = first;
        this.data.last = last;
        this.data.wait = wait;
        this.data.drawTime = drawTime;

	  if(exists(this.frameName, this.uniqueDivName))
	    this.remove();

	  this.uniqueDivName = this.name + (new Date() - 0);

	  frames[this.frameName].root.insertAdjacentHTML('BeforeEnd', '<div ' + this.mouseEvents() + ' style="position: absolute; z-index: 2;" id="' + this.uniqueDivName + '"></div>');  

	  this.setClickable(this.clickable, false);

	  var arrowPoint = getArrowPoint(this.thickness);
	  var drawingLength = this.length - (first?arrowPoint.height:0) - (last?arrowPoint.height:0);

	  var drawingStep = Math.max(1, Math.floor(drawingLength*linePointTimeout/drawTime));

        if(this.sections.length>0)
          this.sections[this.sections.length-1].draw(this.frameName, this.name, color, this.thickness, first, last, drawingStep, wait, this.sections.length-1, drawTypeReverse);

        if(!wait)
          currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Member function create() for Arrow object. Show Arrow into it's pane.
	//----------------------------------------------------------------------------

	function createArrow(color, first, last)
	{
	  this.uniqueDivName = this.name + (new Date() - 0);

	  frames[this.frameName].root.insertAdjacentHTML('BeforeEnd', '<div ' + this.mouseEvents() + ' style="position: absolute; z-index: 2;" id="' + this.uniqueDivName + '"></div>');  

	  this.setClickable(this.clickable, false);

	  var i;

	  if(this.sections.length==1 && first && last)
	  {
          this.sections[0].show(this.frameName, this.uniqueDivName, color, this.thickness, 3);
          return;
	  }

	  if(this.sections.length>0)
	    if(first)
            this.sections[0].show(this.frameName, this.uniqueDivName, color, this.thickness, 1);
	    else
            this.sections[0].show(this.frameName, this.uniqueDivName, color, this.thickness, 0);

	  for(i=1; i<this.sections.length-1; i++)
          this.sections[i].show(this.frameName, this.uniqueDivName, color, this.thickness, 0);

	  if(this.sections.length>0)
	    if(last)
            this.sections[this.sections.length-1].show(this.frameName, this.uniqueDivName, color, this.thickness, 2);
	    else
            this.sections[this.sections.length-1].show(this.frameName, this.uniqueDivName, color, this.thickness, 0);
	}

	//----------------------------------------------------------------------------
	// Member function reset() for Arrow object. 
	// Restore init values of object's attributes.
	//----------------------------------------------------------------------------

	function resetArrow()
	{
	  this.x = this.data.x; 
	  this.y = this.data.y;
	  this.sections = computeSections(this.data.sections, this.data.x, this.data.y, this.thickness);
	  this.clickable = this.data.clickable;
        this.data.lastMethod=NO_LAST_METHOD;
	}

	//----------------------------------------------------------------------------
	// Function computeSections(). Creates sections array from deltas array.
	//----------------------------------------------------------------------------

	function computeSections(deltas, x, y, thickness)
	{
	  var sections = new Array();
	  var i;
	  var p = Math.floor((thickness-1)/2);
	  var dx, dy;

	  for(i=0; i<deltas.length; i++)
	  {
	    dx = deltas[i].dx;
	    dy = deltas[i].dy;
	    if(dx!=0)
	      if(dx>0)
	      {
	        sections[i] = new Section(y-p, x-p, dx+thickness, 0);
	        x = x + dx;
	      }
	      else
	      {
	        sections[i] = new Section(y-p, x+dx-p, dx-thickness, 0);
	        x = x + dx;
	      }
	    else
	      if(dy>0)
	      {
	        sections[i] = new Section(y-p, x-p, 0, dy+thickness);
	        y = y + dy;
	      }
	      else
	      {
	        sections[i] = new Section(y+dy-p, x-p, 0, dy-thickness);
	        y = y + dy;
	      }
	  }

	  return sections;
	}

	//----------------------------------------------------------------------------
	// Function computeLength(). Compute length of arrow given as array of 
	// Section objects.
	//----------------------------------------------------------------------------

	function computeLength(sections)
	{
	  var length = 0;
	  var i = 0;

	  for(i=0; i<sections.length; i++)
	    length += Math.abs(sections[i].dx + sections[i].dy)

	  return length;
	}

//----------------------------------------------------------------------------
// Constructor for Operation
//----------------------------------------------------------------------------

function Operation(operation, delayBefore, asynchronous)
{
  // member attributes
  
  this.operation = operation;
  this.delayBefore = delayBefore;
  this.asynchronous = (asynchronous==null?false:asynchronous);

  this.name = 'operation___' + (new Date() - Math.round(Math.random()*10000));

  // member functions

  this.execute = executeOperation;
  this.executeTimeout = executeTimeout;
}

	//----------------------------------------------------------------------------
	// Member function execute().
	//----------------------------------------------------------------------------

	function executeOperation()
	{ 
	  if(!stopSimulation)
          registerTimeout(this.name, setTimeout(this.operation, this.delayBefore));
	}

	//----------------------------------------------------------------------------
	// Member function executeTimeout().
	//----------------------------------------------------------------------------

	function executeTimeout(timeout)
	{ 
	  if(!stopSimulation)
          registerTimeout(this.name, setTimeout(this.operation, timeout));
	}

//----------------------------------------------------------------------------
// Constructor for Step
//----------------------------------------------------------------------------

function Step(dbStepId, nextStepId, level, nextStepOverId, operations)
{
  // member attributes
  
  if(operations==null)
    this.operations = new Array();
  else
    this.operations = operations;

  this.stepId = 0; // must be initialized to index of this step into steps array, by calling computeSteps method
  this.dbStepId = dbStepId;
  this.nextStepId = nextStepId;
  this.level = level;
  this.nextStepOverId = nextStepOverId;
  this.currentOpId = 0;

  // member functions

  this.executeNext = executeNextStepOperation;
  this.setNextStepOverId = setNextStepOverId;
  this.add = addOperation;
  this.isNextAsynchronous = isNextAsynchronous;
}

	//----------------------------------------------------------------------------
	// Member function executeNext().
	// If wait is false next step is running without waithing between steps,
	// else time between execution of steps is stepTimeout.
	//----------------------------------------------------------------------------

	function executeNextStepOperation(wait, innerCallOnOperationEnd)
	{
	  if(stopSimulation)
	    return;

        if(innerCallOnOperationEnd==null)
          innerCallOnOperationEnd = false;

        if(this.operations.length<=this.currentOpId && innerCallOnOperationEnd) // step end
        {
          stepEnd(); // rise stepEnd event

          if(!auto)
            if(drillDown && currentStep.nextStepOverId!=null && currentStep.nextStepId==null)
            {
              if(!isBreakpoint(steps[currentStep.nextStepOverId]) && !isFinish(steps[currentStep.nextStepOverId]) && steps[currentStep.nextStepOverId]!=null)
                return;
            }
            else
              if(!isBreakpoint(steps[currentStep.nextStepId]) && !isFinish(steps[currentStep.nextStepId]) && steps[currentStep.nextStepId]!=null)
                return;
        }

        if(wait==null)
          wait = true;

        if(this.operations.length>this.currentOpId)
        {
          if(this.currentOpId==0) // step begin
            stepBegin(); // rise stepBegin event

          if(!wait || isBreakpoint(currentStep) || isFinish(currentStep))
            this.operations[this.currentOpId].executeTimeout();
          else
            if(this.currentOpId==0)
              this.operations[this.currentOpId].executeTimeout(stepTimeout);
            else
              this.operations[this.currentOpId].execute();

          this.currentOpId++;

          return;
        }

        // if this is step before "finish();" - i.e. this is real finish
        if(this.nextStepId==null && this.nextStepOverId!=null)
        {
          currentStep = steps[this.nextStepOverId];
          currentStep.executeNext(wait);

          return;
        }

        if(!drillDown && this.nextStepOverId!=null)
        {
          currentStep = steps[this.nextStepOverId];
          if(steps[this.nextStepOverId])
            currentStep.executeNext(wait);
          else
            simulationEnd(); // rise simulationEnd event
        }
        else
          if(this.nextStepId!=null)
          {
            currentStep = steps[this.nextStepId];
            currentStep.executeNext(wait);
          }
          else
            simulationEnd(); // rise simulationEnd event
	}

	//----------------------------------------------------------------------------
	// Member function setNextStepOverId(). 
	// Sets id of next step when drill down is off.
	//----------------------------------------------------------------------------

	function setNextStepOverId(nextStepOverId)
	{
        this.nextStepOverId = nextStepOverId;
	}

	//----------------------------------------------------------------------------
	// Member function add(operation). 
	// Appends operation in the end of operations array.
	//----------------------------------------------------------------------------

	function addOperation(operation)
	{
        if(!this.operations)
          this.operations = new Array();

        this.operations[this.operations.length] = operation;
	}

	//----------------------------------------------------------------------------
	// Member function isNextAsynchronous(operation). 
	//----------------------------------------------------------------------------

	function isNextAsynchronous()
	{
//        this.operations[this.operations.length] = operation;
	}

//----------------------------------------------------------------------------
// Member functions for all objects.
//----------------------------------------------------------------------------

	//----------------------------------------------------------------------------
	// Member function show(). Show HTML object into it's pane.
	//----------------------------------------------------------------------------

	function show(color, first, last)
	{
        this.data.lastMethod = SHOW_METHOD;
        this.data.color = color;
        this.data.first = first;
        this.data.last = last;

	  if(exists(this.frameName, this.uniqueDivName))
	    this.remove();

	  this.create(color, first, last);

        currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Member function hide(). Hide HTML object associated by current element.
	//----------------------------------------------------------------------------

	function hide()
	{
        this.data.lastMethod = HIDE_METHOD;
        this.clickable = this.data.clickable;

	  this.remove();

	  if(lastOverlibObjectName==this.name)
	    hideOver();

	  currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Member function remove(). Remove HTML object associated by current element.
	//----------------------------------------------------------------------------

	function remove()
	{
	  var el = exists(this.frameName, this.uniqueDivName);

	  if(el)
	    el.outerHTML="";
	}

	//----------------------------------------------------------------------------
	// Member function showTimeBlink().
	//----------------------------------------------------------------------------

	function showTimeBlink(timeout, blinkTime, color, first, last)
	{
        this.data.lastMethod = SHOW_TIME_BLINK_METHOD;
        this.data.timeout = timeout;
        this.data.blinkTime = blinkTime;
        this.data.color = color;
        this.data.first = first;
        this.data.last = last;

	  if(exists(this.frameName, this.uniqueDivName))
	    this.remove();

	  this.create(color, first, last);
	  blinkElement(this.frameName, this.uniqueDivName, timeout, blinkTime);
	}

		//----------------------------------------------------------------------------
		// Function blinkElement().
		//----------------------------------------------------------------------------

		function blinkElement(frameName, elementId, timeout, blinkTime)
		{
		  var el = exists(frameName, elementId);

		  if(!el)
		  {
		    currentStep.executeNext(true, true);
		    return;
		  }

		  if(blinkTime>0)
		  {
		    blinkTime = blinkTime - timeout;

		    if(el.style.visibility != "hidden")
		      el.style.visibility = "hidden";
		    else
		      el.style.visibility = "visible";

		    setTimeout("blinkElement('" + frameName + "', '" + elementId + "', '" + timeout + "', '" + blinkTime + "');", timeout);
		  }
		  else
		  {
		    if(el.style.visibility != "visible")
		      el.style.visibility = "visible";

		    currentStep.executeNext(true, true);
		  }
		}

	//----------------------------------------------------------------------------
	// Member function showBlink(). Show blinking object into it's pane.
	//----------------------------------------------------------------------------

	function showBlink(flashTimer, color, first, last)
	{
        this.data.lastMethod = SHOW_BLINK_METHOD;
        this.data.flashTimer = flashTimer;
        this.data.color = color;
        this.data.first = first;
        this.data.last = last;

	  if(exists(this.frameName, this.uniqueDivName))
	    this.remove();

	  this.create(color, first, last);

	  makeFlash(this.frameName, this.uniqueDivName, flashTimer);

        currentStep.executeNext(true, true);
	}

		//----------------------------------------------------------------------------
		// Function makeFlash(). Make element with id elementId blinking by interval
		// flashTimer.
		//----------------------------------------------------------------------------

		function makeFlash(frameName, elementId, flashTimer)
		{
		  var el = exists(frameName, elementId);

		  if(!el)
		    return;

		  if(el.stopBlink)
		  {
	  	    if(el.style.visibility != "visible")
	  	      el.style.visibility = "visible";

		    return;
		  }

	  	  if(el.style.visibility != "hidden")
	  	    el.style.visibility = "hidden";
	  	  else
	  	    el.style.visibility = "visible";

		  setTimeout("makeFlash('" + frameName + "', '" + elementId + "', " + flashTimer + ")", flashTimer);
		}

	//----------------------------------------------------------------------------
	// Member function stop(). Stop blinking effect to object.
	//----------------------------------------------------------------------------

	function stop()
	{
        this.data.lastMethod = STOP_METHOD;

	  var el = exists(this.frameName, this.uniqueDivName);

	  if(el)
	    el.stopBlink = true;

        currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Member function isVisible(). Is this object currently visible?
	//----------------------------------------------------------------------------

	function isVisible()
	{
        if(this.data.lastMethod!=HIDE_METHOD && this.data.lastMethod!=NO_LAST_METHOD)
          return true;

        return false;
	}

	//----------------------------------------------------------------------------
	// Member function setPosition(). Change position of object.
	// If object is visible - last visibility operation continue.
	//----------------------------------------------------------------------------

	function setPosition(top, left)
	{
        this.top=top;
        this.left=left;
        this.x=left;
        this.y=top;

	  if(this.sections)
	    this.sections = computeSections(this.data.sections, this.x, this.y, this.thickness);

	  if(this.isVisible())
        {
	    if(this.data.lastMethod==SHOW_BLINK_METHOD)
	      this.showBlink(this.data.flashTimer, this.data.color, this.data.first, this.data.last);
	    else
	      this.show(this.data.color, this.data.first, this.data.last);
        }
	  else
	    currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Member function setClickable(). Change value of clickable attribute for object.
	// Set this value to visible presentation of object.
	// Param executeNextOperation is for inner call only.
	//----------------------------------------------------------------------------

	function setClickable(clickable, executeNextOperation)
	{
	  this.clickable = (clickable==null || this.onClick_dbStepId==null?false:clickable);

	  var el = exists(this.frameName, this.uniqueDivName);
	  if(el)
	    el.clickable = this.clickable;

	  if(executeNextOperation==null)
	    executeNextOperation = true;

	  if(executeNextOperation)
	    currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Member function setClickableStep().
	// Set values of onClick_dbStepId, clickable, autoPlay attributes.
	//----------------------------------------------------------------------------

	function setClickableStep(onClick_dbStepId, clickable, autoPlay)
	{
	  this.onClick_dbStepId = onClick_dbStepId;
	  this.autoPlay = autoPlay;
	  this.clickable = (clickable==null || this.onClick_dbStepId==null?false:clickable);
	  this.data.clickable = this.clickable;
	}

	//----------------------------------------------------------------------------
	// Member function setHint().
	// Set values of hint attribute.
	//----------------------------------------------------------------------------

	function setHint(hint)
	{
	  this.hint = hint;
	  if(this.hint!=null)        
	    this.hint.frameName = this.frameName;
	}

	//----------------------------------------------------------------------------
	// Member function mouseEvents(). 
	// 1. Correct hint's enviorenment in run time
	// 2. Compose text to onMouseOver and onClick events on object that have onClick_dbStepId attribute.
	// 3. Compose text to onMouseOver and onMouseOut events on object that have Hint attribute.
	// 4. Return events text.
	//----------------------------------------------------------------------------

	function mouseEvents()
	{
	  var onClickText_ObjectClick = '';
	  var onMouseOverText_ObjectClick = '';
	  var onMouseOverText_Hint = '';
	  var onMouseOutText_Hint = '';
	  var i=0;

	  if(this.onClick_dbStepId!=null)
	  {
 	    // get OnClick script
	    onClickText_ObjectClick = '';

	    for(i=currentModel_firstStepId; i<steps.length; i++)
	      if(this.onClick_dbStepId==steps[i].dbStepId)
	      {
	        onClickText_ObjectClick = "parent.stopSimulation=true; parent.stopTimeouts(); parent.resetSteps(); parent.currentStep = parent.steps[" + steps[i].stepId + "]; parent.stopSimulation=false; parent.simulationBegin(); " + (this.autoPlay==1?"parent.auto=true; ":(this.autoPlay==2?"parent.auto=false; ":"")) + "parent.currentStep.executeNext(false);";
	        break;
	      }

	    if(onClickText_ObjectClick!='')
	    {
	      onClickText_ObjectClick = "onclick = \"if(this.clickable){ " + onClickText_ObjectClick + " }\"";
	      onMouseOverText_ObjectClick = "if(this.clickable){ this.style.cursor='Hand'; } else { this.style.cursor='Default'; } ";
	    }
	  }

	  if(this.hint!=null)
	  {
 	    // change hint's enviorenment
	    this.hint.setFrame();
	    eval('top.' + this.hint.name + '=this.hint;');

	    onMouseOverText_Hint = "parent.lastOverlibObjectName = '" + this.name + "';return parent.overlib('" + this.hint.name + "');";
	    onMouseOutText_Hint = "onmouseout = \"return parent.nd();\"";
	  }

	  // compose result

	  var mouseEventsText = '';

	  if(onMouseOverText_ObjectClick=='')
	  {
	    if(onMouseOverText_Hint!='')
	      mouseEventsText = "onmouseover = \"" + onMouseOverText_Hint + "\"";
	  }
	  else
	    if(onMouseOverText_Hint!='')
	      mouseEventsText = "onmouseover = \"" + onMouseOverText_ObjectClick + " " + onMouseOverText_Hint + "\"";
	    else
	      mouseEventsText = "onmouseover = \"" + onMouseOverText_ObjectClick + "\"";

	  mouseEventsText = mouseEventsText + " " + onMouseOutText_Hint + " " + onClickText_ObjectClick;

	  return mouseEventsText;
	}

//----------------------------------------------------------------------------
// Global functions for simulation and all objects.
//----------------------------------------------------------------------------

	//----------------------------------------------------------------------------
	// Function exists(). Return found element with id elementId if it exists 
	// into frame frameName, else - false/null.
	//----------------------------------------------------------------------------

	function exists(frameName, elementId)
	{
	  var el = frames[frameName].document.getElementById(elementId);

	  return el;
	}

	//----------------------------------------------------------------------------
	// Function getArrowPoint(). Return ArrowPoint object from arrowPoint array 
	// for given thickness. If not exists neccessary arrow return last object 
	// from array or null if array is empty.
	//----------------------------------------------------------------------------

	function getArrowPoint(thickness)
	{
	  var i;
	  var j=-1;
	  var proximity = null;

	  for(i=0; i<arrowPoints.length; i++)
          if(arrowPoints[i].forThickness == thickness)
          {
	      j=i;
	      break;
          }
          else
	      if(proximity==null || proximity>=Math.abs(arrowPoints[i].forThickness - thickness))
	      {
	        proximity = Math.abs(arrowPoints[i].forThickness - thickness);
	        j=i;
	      }

	  if(j==-1)
	    return null;

	  return new ArrowPoint(thickness, 
                              Math.floor(arrowPoints[j].width*thickness/arrowPoints[j].forThickness), 
                              Math.floor(arrowPoints[j].height*thickness/arrowPoints[j].forThickness), 
                              arrowPoints[j].srcLeft, 
                              arrowPoints[j].srcUp);
	}

	//----------------------------------------------------------------------------
	// Function setStepTimeout(). Sets new values of stepTimeout and revealTransTime.
	//----------------------------------------------------------------------------

	function setStepTimeout(stepTimeoutNew)
	{
	  stepTimeout = stepTimeoutNew;

	  revealTransTime = Math.min(4.0, stepTimeout/2000.0);
	}

	//----------------------------------------------------------------------------
	// Function resetSteps(). Sets init values of currentOpId to all steps.
	//----------------------------------------------------------------------------

	function resetSteps()
	{
	  for(var i=0; i<steps.length; i++)
	    steps[i].currentOpId = 0;
	}

	//----------------------------------------------------------------------------
	// Function computeSteps(). Sets stepId attributes to steps.
	//----------------------------------------------------------------------------

	function computeSteps()
	{
	  for(var i=0; i<steps.length; i++)
	  {
	    if(isFinish(steps[steps[i].nextStepId]))
	    {
	      if(steps[i].level==1 && steps[i].level==steps[steps[i].nextStepId].level)
	        steps[i].nextStepOverId = null;
	      else
	        steps[i].nextStepOverId = steps[steps[i].nextStepId].nextStepOverId;

	      steps[i].nextStepId = null;
	    }

	    steps[i].stepId = i;
	  }
	}

	//----------------------------------------------------------------------------
	// Function isBreakpoint().
	// Return true if step is breakpoint step, else - false.
	//----------------------------------------------------------------------------

	function isBreakpoint(step)
	{
	  if(step!=null && step.operations && step.operations[0])
	    if(step.operations[0].operation=="breakpoint();")
	      return true;

	  return false;
	}

	//----------------------------------------------------------------------------
	// Function isFinish().
	// Return true if step is finish step, else - false.
	//----------------------------------------------------------------------------

	function isFinish(step)
	{
	  if(step!=null && step.operations && step.operations[0])
	    if(step.operations[0].operation=="finish();")
	      return true;

	  return false;
	}

	//----------------------------------------------------------------------------
	// Function resetObjects(). Sets initial values of parameters to all 
	// simulation's objects.
	//----------------------------------------------------------------------------

	function resetObjects()
	{
	  var j=0, i=0;
	  var obj;
	  var operations;

	  for(j=0; j<steps.length; j++)
	  {
	    operations = steps[j].operations;
	    for(i=0; i<operations.length; i++)
	    {
	      obj = getSimObject(operations[i].operation); 
	      if(obj && obj.reset)
              obj.reset();
	    }
	  }
	}

	//----------------------------------------------------------------------------
	// Function resetFrame(). Clear objects from frame frameName and execute all 
	// operations from step by index initStepId related to this frame.
	//----------------------------------------------------------------------------

	function resetFrame(frameName, initStepId, executeNext)
	{
	  var el = exists(frameName, 'root');

	  if(el!=null)
	    el.innerHTML="";

	  if(steps[initStepId]==null)
	    return;

	  // if overlib is used, hide over object on model's hiding.
	  hideOver();

	  var j=0, i=0;
	  var obj;
	  var operations = steps[initStepId].operations;
	  var stopSimulationTemp = stopSimulation;
        stopSimulation = true;

	  var currentStepNumber = currentStep.stepId;
	  var currentOpId = currentStep.currentOpId;

	  // reset GUI frame
	  if(frameName==guiFrameName && frames[guiFrameName])
	    frames[guiFrameName].location.replace(standartGUIPage);

	  for(i=0; i<operations.length; i++)
	  {
	    obj = getSimObject(operations[i].operation); 
	    if(obj!=null)
	    {
	      if(frameName==guiFrameName && obj.type!=null && obj.type=="GUI")
	      {
              eval(operations[i].operation);
              continue;
	      }

            if(obj.frameName!=null && obj.frameName==frameName)
            {
              obj.reset();
              eval(operations[i].operation);
              continue;
            }

            if(obj.type!=null && obj.type=="Model")
            {
	        for(j=0; j<obj.objects.length; j++)
	          if(obj.objects[j].type=="PaneBackground" && obj.objects[j].frameName==frameName)
	            obj.objects[j].show();
            }
	    }
	  }

        stopSimulation = stopSimulationTemp;
	  currentStep = steps[currentStepNumber];
	  currentStep.currentOpId = currentOpId;

	  if(executeNext==null || executeNext)
	    currentStep.executeNext(true, true);
	}

		//----------------------------------------------------------------------------
		// Function getSimObject(). Parse string paameter and return variable 
		// that represent SIM object - name of variable is the string before point sign.
		// If variable not exists return null.
		//----------------------------------------------------------------------------

		function getSimObject(operation)
		{
	        var objectName = operation.substr(0, operation.indexOf('.')); 
	        return eval(objectName);
		}

	//----------------------------------------------------------------------------
	// Function resizeFrame(frameName, size). Resize frame frameName.
	//----------------------------------------------------------------------------

	function resizeFrame(frameName, size, executeNext)
	{ 
	  if(frameName=='processFrame')
	    if(size=='*')
	      outerFrameset.rows = '*, 0';
	    else
	      outerFrameset.rows = '' + size + ', *';

	  if(frameName=='architectureFrame')
	    if(size=='*')
	      innerFrameset.cols = '*, 0';
	    else
	      innerFrameset.cols = '' + size + ', *';

	  if(frameName=='guiFrame')
	    if(size=='*')
	      innerFrameset.cols = '0, *';
	    else
	      innerFrameset.cols = '*, ' + size;

	  if(executeNext==null || executeNext)
	    currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Function startSimulation(). Start execution of simulation.
	//----------------------------------------------------------------------------

	function startSimulation()
	{ 
        top.menuFrame.simButtonsShow(playButtonActive, stopButtonActive, skipButtonActive, nextButtonActive, rewindButtonActive, drillDownButtonActive, delayButtonActive);

	  computeSteps();
	  resetSteps();
	  resetObjects();
	  lastOverlibObjectName = "";
	  currentStep = steps[0];

        simulationBegin(); // rise simulationBegin event

	  currentStep.executeNext(false);
	}

	//----------------------------------------------------------------------------
	// Function simulationBegin(). Must be called when simulation is begining.
	// Rise simulationBegin event.
	//----------------------------------------------------------------------------

	function simulationBegin()
	{ 
        if(parent.onSimulationBegin) // rise simulationBegin event
          parent.onSimulationBegin();

        stopped = false;
	}

	//----------------------------------------------------------------------------
	// Function simulationEnd(). Must be called when simulation is ending.
	// Rise simulationEnd event.
	//----------------------------------------------------------------------------

	function simulationEnd()
	{ 
        if(parent.onSimulationEnd) // rise simulationEnd event
          parent.onSimulationEnd();

        stopped = true;
	}

	//----------------------------------------------------------------------------
	// Function stepBegin(). Must be called when step is begining.
	// Rise stepBegin event.
	//----------------------------------------------------------------------------

	function stepBegin()
	{ 
        if(parent.onStepBegin) // rise stepBegin event
          parent.onStepBegin();

        stopped = false;
	}

	//----------------------------------------------------------------------------
	// Function stepEnd(). Must be called when step is ending.
	// Rise stepEnd event.
	//----------------------------------------------------------------------------

	function stepEnd()
	{ 
        if(parent.onStepEnd) // rise stepEnd event
          parent.onStepEnd();

        stopped = true;
	}

	//----------------------------------------------------------------------------
	// Function breakpoint(). Temporary stop execution of simulation.
	//----------------------------------------------------------------------------

	function breakpoint()
	{
	  var i;

	  auto=false; 
	  top.menuFrame.simClick(top.menuFrame.stop);

	  var stopSimulationTemp = stopSimulation;

	  if(skipClicked) // some operations are performed only wneh skip button is clicked
	  {
	    skipClicked = false;

          stopSimulation = true;

	    for(i=currentStep.currentOpId; i<currentStep.operations.length; i++)
            eval(currentStep.operations[i].operation);

          stopSimulation = stopSimulationTemp;
	  }
	  else
	    currentStep.currentOpId = currentStep.operations.length;

	  currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Function finish(). Ends execution of simulation.
	//----------------------------------------------------------------------------

	function finish()
	{
	  currentStep.currentOpId = currentStep.operations.length; // ignore other operations from this step

	  if(currentStep.nextStepOverId!=null)
	    currentStep = steps[currentStep.nextStepOverId];

	  currentStep.executeNext(false);
	}

	//----------------------------------------------------------------------------
	// Function registerTimeout(). Add or replace timeout to object timerIds.
	//----------------------------------------------------------------------------

	function registerTimeout(timerIdName, setTimeoutFunction)
	{
	  var timerId = setTimeoutFunction;
	  eval("timerIds." + timerIdName + " = timerId;");
	}

	//----------------------------------------------------------------------------
	// Function stopTimeouts(). Stop all runing timeouts registered to object timerIds.
	//----------------------------------------------------------------------------

	function stopTimeouts()
	{ 
	  var i;

	  for(i in timerIds)
	    clearTimeout(timerIds[i]);
	}

	//----------------------------------------------------------------------------
	// Function scrollFrame(). Scroll content of frame frameName.
	//----------------------------------------------------------------------------

	function scrollFrame(frameName, x, y, pixel_measure, executeNext)
	{ 
	  var fr = frames[frameName].document.body;

	  if(pixel_measure==null)
	    pixel_measure=true;

	  if(pixel_measure)
	  {
	    fr.scrollTop = y;
	    fr.scrollLeft = x;
	  }
	  else
	  {
	    fr.scrollTop = Math.round(fr.scrollHeight*y/100);
	    fr.scrollLeft = Math.round(fr.scrollWidth*x/100);
	  }

	  if(executeNext==null || executeNext)
	    currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Function pane(). Perform same actions as methods resetFrame, resizeFrame 
	// and scrollFrame in this order if necessary for them params are given.
	//----------------------------------------------------------------------------
	
	function pane(frameName, reset, initStepId, size, x, y, pixel_measure)
	{
        if(reset!=null && reset)
          resetFrame(frameName, initStepId, false);

        if(size!=null && size!='')
          resizeFrame(frameName, size, false);

        if(x!=null && y!=null)
          scrollFrame(frameName, x, y, pixel_measure, false);

	  currentStep.executeNext(true, true);
	}

	//----------------------------------------------------------------------------
	// Function hideOver(). If overlib is used, hide over object on model's hiding.
	//----------------------------------------------------------------------------

	function hideOver()
	{
	  if(nd)
	    nd();
	}

	//----------------------------------------------------------------------------
	// Function skip(). Perform 'skip' operation to current simulation.
	//----------------------------------------------------------------------------

	function skip()
	{
	  var stepId = currentStep.stepId;
	  var i = 0;

	  skipClicked = true;

	  while(true)
	  {
 	    // go over subscript or return to parent script
 	    if(steps[stepId].nextStepOverId!=null)
 	    {
 	      // go over subscript
 	      if(steps[stepId].level==steps[steps[stepId].nextStepOverId].level)
 	      {
	        stepId = steps[stepId].nextStepOverId;

	        if(isBreakpoint(steps[stepId]))
 	        {
	          stopSimulation=true;
	          stopTimeouts();
	          stopSimulation=false;

	          currentStep = steps[stepId];
 	          currentStep.executeNext(false);	    

	          return;
 	        }
 	      }
 	      else // return to parent script from subscript - first way - with finish step
 	        if(steps[stepId].level>steps[steps[stepId].nextStepOverId].level)
 	        {
	          stepId = steps[stepId].nextStepOverId;

	          stopSimulation=true;
	          stopTimeouts();
	          stopSimulation=false;

	          currentStep = steps[stepId];
	          currentStep.executeNext(false);

	          return;
 	        }
 	        else  // next step over id is drillDown and must get previos step (show)
 	          for(i=0; i<steps.length; i++)
 	            if(steps[i].nextStepId==steps[steps[stepId].nextStepOverId].stepId)
 	            {
	              stepId = steps[i].stepId;
 	              break;
 	            }
 	    }
 	    else
 	    {
 	      if(steps[stepId].nextStepId!=null)
 	      {
 	        // return to parent script from subscript - second way - without finish step
 	        if(steps[stepId].level!=steps[steps[stepId].nextStepId].level)
 	        {
	          stepId = steps[stepId].nextStepId;

	          stopSimulation=true;
	          stopTimeouts();
	          stopSimulation=false;

	          currentStep = steps[stepId];
	          currentStep.executeNext(false);

	          return;
 	        }

	        stepId = steps[stepId].nextStepId;

	        if(isBreakpoint(steps[stepId]))
 	        {
	          stopSimulation=true;
	          stopTimeouts();
	          stopSimulation=false;

	          currentStep = steps[stepId];
 	          currentStep.executeNext(false);	    

	          return;
 	        }
 	      }
 	      else   // end of root script of simulation
	      {
	        skipEnabled = false;
	        top.menuFrame.simButtons[top.menuFrame.skipButton].setActive(false);
	        return;
	      }
 	    }
	  }
	}